from .gpt_classes.api_selection.ApiBuilder import ApiManager
from .gpt_classes.model_selection.ModelBuilder import ModelManager
from .gpt_classes.prompt_selection.PromptBuilder import PromptManager
from .gpt_classes.instruction_selection.InstructionBuilder import InstructionManager
from .gpt_classes.response_selection.ResponseBuilder import ResponseManager
import asyncio
def get_query(response_mgr):
    return asyncio.run(response_mgr.initial_query())
def make_general_query(prompt=None,
                       api_key=None,
                       env_path=None,
                       completion_percentage=50,
                       model=None,
                       instructions=None,
                       instruction_bools=None,
                       data=None):
    api_mgr = ApiManager(api_key=api_key,env_path=env_path)
    model_mgr = ModelManager(input_model_name=model)
    inst_mgr = InstructionManager(instructions=instructions,instruction_bools=instruction_bools)
    prompt_mgr = PromptManager(request_data=prompt,prompt_data=data,completion_percentage=completion_percentage,instruction_mgr=inst_mgr,model_mgr=model_mgr)
    response_mgr = ResponseManager(prompt_mgr=prompt_mgr,api_mgr=api_mgr)
    return get_query(response_mgr)




from ..dependencies import *
def query_event_check(self) -> bool:
    if self.event == "-SUBMIT_QUERY-":
        if not self.submission_in_progress:
            self.submission_in_progress = True
            self.get_new_api_call_name()
            # Submit the coroutine to the event loop
            future = asyncio.run_coroutine_threadsafe(self.submit_query(), self.loop)
            # Optionally, handle the result or exceptions
            future.add_done_callback(self.handle_future_result)
        else:
            self.logger.info("Submission already in progress.")
        self.event = None
    else:
        return False
    return True

def handle_future_result(self, future):
    try:
        result = future.result()
        # Process the result if needed
    except Exception as e:
        self.logger.error(f"Error in async task: {e}")
def submit_event(self):
    logger.info(f"submit_event..")
    self.get_new_api_call_name()
    self.start_query = False
    self.updated_progress = False
    self.response_mgr.re_initialize_query()
    asyncio.run(self.submit_query())
    self.submission_in_progress = False  # Reset the flag when done
async def submit_query(self) -> None:
    logger.info(f"submit_query..")
    try:
        # Update the GUI components from the main thread using a thread-safe method
        self.window.write_event_value('-DISABLE_SUBMIT_BUTTON-', True)

        self.dots = '...'
        self.start_query = False
        if self.test_bool:
            self.latest_output = [safe_read_from_json(self.test_file_path)]
        else:
            self.latest_output = await self.get_query() or []
        self.output_list.append(self.latest_output)
        self.logger.info(f"recent output_list: {self.output_list}")
        self.update_progress_chunks(done=True)
        self.update_last_response_file()
        self.update_text_with_responses()
        # Re-enable the submit button
        self.window.write_event_value('-ENABLE_SUBMIT_BUTTON-', True)
        if not self.window_mgr.get_values()['-REUSE_CHUNK_DATA-']:
            self.update_prompt_data('')
        self.response = False
    finally:
        self.submission_in_progress = False
    
    
async def get_query(self) -> None:
    logger.info(f"get_query..")
    self.response = await self.response_mgr.initial_query()
    return self.response



from getAll import *
import asyncio
def get_raw_response_directory():
    return os.path.join(os.getcwd(),"response_data")
def get_latest_file(directory):
    files = glob.glob(f"{directory}/*")
    latest_file = max(files, key=os.path.getmtime)
    return latest_file
def get_last_created_response_file_path():
    raw_response_directory = get_raw_response_directory()
    return get_latest_file(raw_response_directory)
def read_previous_query():
    recent_file_path = get_last_created_response_file_path()
    return safe_read_from_json(recent_file_path)
def parse_previous_query():
    response_mgr = ResponseFileCollator()
    return self.ordered_data[0]
def get_db_query():
    file_path = get_last_created_response_file_path()
    return generate_query_from_recent_response(file_path)
def get_prompt_manager(prompt=None,
                       prompt_data=None,
                           request_data=[''],
                           instruction_data=None,
                           chunk_token_distribution_number=None,
                           completion_percentage=None,
                           instruction_mgr=None,
                           notation=None,
                           model_mgr=None,
                           chunk_number=None,
                           chunk_type=None,
                           model=None):
  prompt = prompt or ''
  prompt_data = prompt_data or []
  request_data = request_data or [prompt]
  instruction_data = instruction_data or []
  chunk_token_distribution_number = chunk_token_distribution_number or 0
  completion_percentage = completion_percentage or 40
  chunk_number=chunk_number or 0
  model_mgr = model_mgr or get_model_mgr(model=model)
  instruction_mgr = instruction_mgr or get_instruction_manager(bool_list = [],instructions={})
  chunk_type = chunk_type or "CODE"
  return PromptManager(prompt_data=[prompt],
                           request_data=[''],
                           instruction_data=[],
                           chunk_token_distribution_number=0,
                           completion_percentage=40,
                           instruction_mgr=instruction_mgr,
                           notation=None,
                           model_mgr=get_model_mgr(model=model),
                           chunk_number=chunk_number,
                           chunk_type=chunk_type)
class setup_module(env_key=None,
                   env_path=None,
                   model=None,
                   prompt=None,
                   prompt_percentage=None,
                   prompt_data=[],
                   tableName=None,
                   additional_instructions=None,
                   instruction_bools=None):
    def __init__(self):
        if tableName:
            prompt_data.append(get_instruction_from_tableName(tableName=tableName))
            instruction_bools['database_query']=True
        self.api_mgr = ApiManager(env_key=env_key,env_path=env_path)
        self.model_mgr = ModelManager(input_model_name = model)
        self.inst_mgr = InstructionManager(instructions,instruction_bools)
        self.prompt_mgr = get_prompt_manager(prompt=prompt,prompt_data=prompt_data,instruction_mgr=inst_mgr,model_mgr=model_mgr,instruction_data=instruction_mgr.instructions)
        self.resp_mgr = ResponseManager(prompt_mgr=prompt_mgr,api_mgr=api_mgr)
    def calculate_extra_tokens(self,prompt=None):
        self.extra_tokens = (self.model_mgr.selected_max_tokens*prompt_percentage) - num_tokens_from_string(prompt_mgr.create_prompt())
        return self.extra_tokens
    async def make_query():
        response_mgr = ResponseManager(prompt_mgr=prompt_mgr,api_mgr=api_mgr)
        response = await response_mgr.initial_query()
        return get_updated_response_content(response)

def make_query(module_mgr):
    return asyncio.run(module_mgr.make_query())
setup_module(prompt='ji')



from abstract_database import *
from abstract_security import *
from .responseContentParser import get_last_created_response_file_path,get_any_data
import psycopg2
import os
import pandas as pd

class DatabaseManager:
    def __init__(self, env_path):
        # Load environment configuration
        self.env_path = env_path
        self.dbname = get_env_value(key="abstract_ai_dbname", path=env_path)
        self.user = get_env_value(key="abstract_ai_user", path=env_path)
        self.password = get_env_value(key="abstract_ai_password", path=env_path)
        self.host = get_env_value(key="abstract_ai_host", path=env_path)
        self.port = int(get_env_value(key="abstract_ai_port", path=env_path))
        self.table_config_path = get_env_value(key="abstract_ai_table_configuration_file_path", path=env_path)
        
    def connect_db(self):
        """Establish a connection to the database."""
        try:
            return psycopg2.connect(
                dbname=self.dbname,
                user=self.user,
                password=self.password,
                host=self.host,
                port=self.port
            )
        except psycopg2.OperationalError as e:
            print(f"Unable to connect to the database: {e}")
            sys.exit(1)

    def get_table_configuration(self, file_path=None):
        """Retrieve table configuration from a file."""
        table_configuration_file_path = file_path or self.table_config_path
        try:
            return safe_read_from_json(table_configuration_file_path)
        except Exception as e:
            print(f"No table config file path found: {e}")
            return []

    def get_dict_from_config(self, tableName, file_path=None):
        """Retrieve a table configuration by table name."""
        for config in self.get_table_configuration(file_path=file_path):
            if config.get('tableName').lower() == tableName.lower():
                return config

    def get_table_names(self, file_path=None):
        """Retrieve all table names from the configuration file."""
        return [config.get('tableName') for config in self.get_table_configuration(file_path=file_path)]

    def get_first_row_as_dict(self, tableName=None):
        """Fetch the first row of data from the specified table and return as a dictionary."""
        tableName = tableName or get_env_value(key="abstract_ai_table_name", path=self.env_path)
        query = f"SELECT * FROM {tableName} ORDER BY id ASC LIMIT 1;"

        conn = self.connect_db()
        cur = conn.cursor()
        try:
            cur.execute(query)
            first_row = cur.fetchone()
            col_names = [desc[0] for desc in cur.description]

            if first_row:
                return dict(zip(col_names, first_row))
            return None
        except psycopg2.Error as e:
            print(f"Error fetching the first row: {e}")
            return None
        finally:
            cur.close()
            conn.close()

    def get_instruction_from_tableName(self, tableName=None):
        """Get instructions based on table configuration and data."""
        tableName = tableName or get_env_value(key="abstract_ai_table_name", path=self.env_path)
        table_samples = []
        table_samples.append({"DATABASE_CONFIG": self.get_dict_from_config(tableName), "explanation": "Database Table Configuration."})

        data = self.get_first_row_as_dict(tableName)
        if data:
            table_samples.append({"ACTUAL_DATABASE_ROW": data, "explanation": f"First row of data from table {tableName} returned as a dictionary."})

            value_keys = {column: get_value_keys(value) for column, value in data.items()}
            table_samples.append({"VALUE_KEYS": value_keys, "explanation": "Type Values for the Values in the Database SCHEMA."})

            table_samples.append({"AVAILABLE_FUNCTION_FOR_FILTERING": self.get_filtering_function(), "explanation": "Available function for filtering the database."})

        return table_samples

    def get_filtering_function(self):
        """Return a string representing the available function for filtering the database."""
        return """def search_multiple_fields(query):
    conn = connect_db()
    cur = conn.cursor()
    try:
        cur.execute(query)
        results = cur.fetchall()
        return results
    except psycopg2.Error as e:
        print(f"Error querying JSONB data: {e}")
    finally:
        cur.close()
        conn.close()
"""

    def search_multiple_fields(self, query,**kwargs):
        """Search the database using a query."""
        conn = self.connect_db()
        cur = conn.cursor()
        try:
            cur.execute(query)
            return cur.fetchall()
        except psycopg2.Error as e:
            print(f"Error querying JSONB data: {e}")
        finally:
            cur.close()
            conn.close()

    def save_to_excel(self, rows, file_path="output.xlsx"):
        """Save query results to an Excel file."""
        excel_data = []
        if rows:
            for row in rows:
                row = list(row) if isinstance(row, tuple) else row
                excel_data.append(flatten_json(row, parent_key='', sep='_'))
            df = pd.DataFrame(excel_data)
            safe_excel_save(df, file_path)

    def get_query_save_to_excel(self, database_query, file_path="output.xlsx"):
        """Fetch query results and save them to an Excel file."""
        result = self.search_multiple_fields(**database_query)
        self.save_to_excel(result, file_path=file_path)
        return file_path

    def generate_query_from_recent_response(self, file_path):
        """Generate a query based on the most recent response and save the result to an Excel file."""
        response_content = get_response_content(file_path)
        database_query = response_content.get('database_query')
        if database_query:
            title = response_content.get('generate_title')
            new_directory = os.path.join(os.path.dirname(file_path), 'queries')
            os.makedirs(new_directory, exist_ok=True)

            new_file_path = os.path.join(new_directory, f"{title}.xlsx")
            self.get_query_save_to_excel(database_query, file_path=new_file_path)
            return new_file_path

    def get_db_query(self):
        """Get the latest database query and save the result."""
        file_path = get_latest_response()
        return self.generate_query_from_recent_response(file_path)
def ensure_db_manager(db_mgr=None,env_path=None):
    return db_mgr or DatabaseManager(env_path = env_path)
def generate_query_from_recent_response(file_path=None,db_mgr=None,env_path =None):
        """Generate a query based on the most recent response and save the result to an Excel file."""
        db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path)
        database_query = get_any_data(file_path=file_path,key_value='database_query')
        if database_query:
            response_content = get_response_content(file_path=file_path)
            title = response_content.get('generate_title')
            new_directory = os.path.join(os.path.dirname(file_path), 'queries')
            os.makedirs(new_directory, exist_ok=True)
            new_file_path = os.path.join(new_directory, f"{title}.xlsx")
            db_mgr.get_query_save_to_excel(database_query, file_path=new_file_path)
            return new_file_path
def get_db_query(db_mgr=None,env_path = None,file_path=None):
    """Get the latest database query and save the result."""
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path)
    file_path = file_path or get_last_created_response_file_path()
    return generate_query_from_recent_response(file_path=file_path,db_mgr=db_mgr,env_path=env_path)
def get_instruction_from_tableName(tableName=None,db_mgr=None,env_path = None,file_path=None):
    """Get the latest database query and save the result."""
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path)
    return db_mgr.get_instruction_from_tableName(tableName=tableName)
def search_multiple_fields(self, query,tableName=None,db_mgr=None,env_path = None,file_path=None,**kwargs):
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path)
    return db_mgr.search_multiple_fields(query)



from abstract_utilities import get_actual_number,eatAll
from abstract_utilities.type_utils import is_bool
def get_dict_from_instruction(instructions):
    new_instructions={}
    if instructions:
        count = 0
        if isinstance(instructions,str):
            new_instructions[f"additional_instructions_{count}"]={"instruction":instructions,"example":"","default":True}
        elif isinstance(instructions,list):
            for instruction in instructions:
                if isinstance(instruction,str):
                    new_instructions[f"additional_instructions_{count}"] = {"instruction":instruction,"example":"","default":True}
                    count +=1
                if isinstance(instruction,dict):
                    new_instruction = {"instruction":"","example":"","default":True}
                    new_instruction.update(instruction)
                    new_instructions[f"additional_instructions_{count}"] = new_instruction
                    count +=1
        elif isinstance(instructions,dict):
            for key,value in instructions.items():
                new_instruction = {"instruction":"","example":"","default":True}
                new_instruction.update(instructions[key])
                new_instructions[key] = new_instruction
    return new_instructions
class InstructionManager:
    def __init__(self,instructions=None,instruction_bools={})->None:
        self.default_instructions = {
                    "api_response": {
                        "instruction": "Place response to prompt here.",
                        "example": "This is the reply to your request.",
                        "default":True},
                    "additional_responses": {
                        "instruction": "Marking 'True' initiates a loop which continues to send the current chunk's prompt until the module returns a 'False' value.",
                        "example": "false",
                        "default":False},
                    "generate_title": {
                        "instruction": "Generate a short and concise title for this query, using underscores instead of spaces and avoiding special characters. The title should not exceed 200 characters to ensure compatibility with file systems.",
                        "example": "Short_Title_for_Query",
                        "default":True},
                    "database_query":{
                        "instruction": "the given database Schema,variables,values and function are provided in the prompt data; please derive the most appropriate inputs for a database query from the context and assertons of the request. place all inputs for the function within this respons key",
                        "example": {"query":"""SELECT dnc_data FROM dncdata WHERE dnc_data ->> 'Data_Subscriber_First_Name' = 'Karen';"""},
                        "default":False},
                    "notation": {
                        "instruction": "notation is a module end  functionality that allows the module, you, to to preserve relevant information or context for subsequent prompts; allowing for communication between modules throughout the query iterations. ",
                        "example": "Selecting additional responses due to insufficient completion tokens.",
                        "default":False},
                    "suggestions": {
                        "instruction": "This parameter allows the module to provide suggestions for improving efficiency in future prompt sequences.",
                        "example": "Consider batching queries to reduce server load",
                        "default":False},
                    "abort": {
                        "instruction": "If you cannot fulfill the request, respond with this value as 'True'. Leave a detailed reason as to why the query stream was aborted in 'suggestions'",
                        "example": "False",
                        "default":False},
                    "prompt_as_previous": {
                        "instruction": "This is a user-end declaration. If this is visible, the request portion of the prompt will change to include previous response data, if needed",
                        "example": "True",
                        "default":False},
                    "request_chunks": {
                        "instruction": "Request to prompt again the previous chunk data. If selected, the query will iterate once more with the previous data chunk included in the prompt.",
                        "example": "False",
                        "default":False},
                    "token_adjustment": {
                        "instruction": "Suggest percentage adjustments, between -100 up to 100, for the future token allotment. If it will provide better results to increase or decrease the future allotment, place a number.",
                        "example": "0",
                        "default":False},
                    }
        self.instructions = [[]]
        self.default_instructions.update(get_dict_from_instruction(instructions))
        for key,value in instruction_bools.items():
            self.default_instructions[key]['default'] = value
        self.update_instructions(0,**{key:self.default_instructions[key] for key in self.default_instructions if self.default_instructions[key].get('default')})
    def get_instructions(self,instruction_number:int=None)->dict:
        if len(self.instructions)==0:
            self.add_instructions()
        instruction_number = instruction_number or -1
        return self.instructions[instruction_number]
    def update_default_instructions(self,data):
        self.default_instructions.update(get_dict_from_instruction(data))
        self.update_instructions(0,**{key:self.default_instructions[key] for key in self.default_instructions if self.default_instructions[key].get('default')})
    def get_instructions_bools(self,parameters:dict)->dict:
        for parameter,value in parameters.items():
            parameters[parameter] = True if value else False
        return parameters
    
    def get_instructions_text_values(self,instruction_bools:dict,parameters:dict)->dict:
        parameter_text_values = {}
        for parameter,value in instruction_bools.items():
            if value:
                parameter_text_values[parameter] = self.default_instructions[parameter]['instruction'] if is_bool(parameters[parameter]) else  parameters[parameter]
        return parameter_text_values
    def get_instructions_text(self,instructions_js:dict)->dict:
        """
        Retrieves instructions for the conversation.

        Returns:
            None
        """
        instructions = ""
        example_format={}
        if instructions_js:
            instructions = "your response is expected to be in JSON format, while any boool responses are to be lowercase with no quotations surrounding their values, with the keys as follows:\n\n"
            i=0
            for key in instructions_js.keys():
                if key in self.default_instructions.keys():
                    instructions+=f'''{i}) {key} - """{eatAll(instructions_js[key],['"'])}"""\n'''
                    example_format[key]=self.default_instructions[key]['example']
                    i+=1
            instructions += '\nbelow is an example of the expected json dictionary response format, with the default inputs:\n' + str(example_format).replace("'",'"')
        return instructions
    def add_instructions(self,all_true:bool=False,**kwargs)->None:
        print('adding instructions')
        if len(self.instructions)==0:
            instruction_display,instruction_bools,instructions_text_values,instructions_text=self.get_instructions_values(all_true,**kwargs)
            last_instructions = {'instruction_display':instruction_display,'instructions_bools':instruction_bools,"instructions_text_values":instructions_text_values,"instructions_text":instructions_text,"text":instructions_text}
        else:
            last_instructions = self.instructions[-1]
        self.instructions.append(last_instructions)
        
    def get_instructions_values(self,all_true:bool= False,**kwargs)->(bool,dict,dict,str):
        parameters = {}
        for parameter in self.default_instructions.keys():
            value = kwargs.get(parameter,False) if all_true == False else True
            parameters[parameter]=value
        instruction_display=kwargs.get("instruction_display",True)    
        instruction_bools = self.get_instructions_bools(parameters)
        instructions_text_values= self.get_instructions_text_values(instruction_bools,parameters)
        instructions_text = self.get_instructions_text(instructions_js=instructions_text_values)
        return instruction_display,instruction_bools,instructions_text_values,instructions_text
    
    def update_instructions(self,instruction_number:int=0,**kwargs)->None:
        
        instruction_display,instruction_bools,instructions_text_values,instructions_text=self.get_instructions_values(**kwargs)
        self.instructions[instruction_number]={'instruction_display':instruction_display,'instructions_bools':instruction_bools,"instructions_text_values":instructions_text_values,"instructions_text":instructions_text,"text":instructions_text}



from abstract_utilities import get_actual_number,eatAll
from abstract_utilities.type_utils import is_bool
from abstract_utilities import get_actual_number
from abstract_utilities.type_utils import is_bool
def get_instructions_list_to_dict(obj)->dict:
    if not isinstance(obj,dict):
        obj = {key: value[key] for value in obj for key in value}
    return obj
def get_instructions_dict_to_list(obj)->list:
    if not isinstance(obj,list):
        obj = [{key:value} for key,value in instructions_dict.items()]
    return obj
class InstructionManager:
    def __init__(self)->None:
        self.default_instructions = {
            "api_response": {
                "instruction": "Place response to prompt here.",
                "example": "This is the reply to your request.",
                "default":True},
            "additional_responses": {
                "instruction": "Marking 'True' initiates a loop which continues to send the current chunk's prompt until the module returns a false value.",
                "example": "false",
                "default":False},
            "generate_title": {
                "instruction": "Generate a short and concise title for this query, using underscores instead of spaces and avoiding special characters. The title should not exceed 30 characters to ensure compatibility with file systems.",
                "example": "Short_Title_for_Query",
                "default":True},
            "notation": {
                "instruction": "notation is a module end  functionality that allows the module, you, to to preserve relevant information or context for subsequent prompts; allowing for communication between modules throughout the query iterations. ",
                "example": "Selecting additional responses due to insufficient completion tokens.",
                "default":True},
            "suggestions": {
                "instruction": "This parameter allows the module to provide suggestions for improving efficiency in future prompt sequences.",
                "example": "Consider batching queries to reduce server load",
                "default":True},
            "abort": {
                "instruction": """If you cannot fulfill the request, respond with this value as true. Leave a detailed reason as to why the query stream was aborted in suggestions""",
                "example": "False",
                "default":False},
            "prompt_as_previous": {
                "instruction": "This is a user-end declaration. If this is visible, the request portion of the prompt will change to include previous response data, if needed",
                "example": 'true',
                "default":False},
            "request_chunks": {
                "instruction": "Request to prompt again the previous chunk data. If selected, the query will iterate once more with the previous data chunk included in the prompt.",
                "example": "false",
                "default":False},
            "token_adjustment": {
                "instruction": "Suggest percentage adjustments, between -100 up to 100, for the future token allotment. If it will provide better results to increase or decrease the future allotment, place a number.",
                "example": "0",
                "default":False}
            }
        self.instruction_number = 0
        self.instructions=[]
       
    def get_default_instructions(self,instruction_number=None):
        instructions = []
        instructions.append([{key:self.default_instructions.get(key)} for key in self.default_instructions if self.default_instructions.get(key).get("default")])
        self.update_instruction(instructions,instruction_number=instruction_number)
        return get_instructions_dict(self)
    def get_instructions_list(self):
        return get_instructions_dict_to_list(self.instructions[self.instruction_number])
    def get_instructions_dict(self):
        return get_instructions_list_to_dict(self.instructions[self.instruction_number])
    def update_instruction(self,instruction,instruction_number=None):
        self.instruction_number=instruction_number or self.instruction_number
        self.instructions[self.instruction_number] = instruction
    def get_instructions(self,instruction_number:int=None)->dict:
        if len(self.instructions)==0:
            self.add_instructions()
        instruction_number = instruction_number or -1
        return self.instructions[instruction_number]
    def update_instructions_bools(self,**kwargs)->dict:
        instructions = self.get_instructions_dict()
        for key,value in instructions.items():
            if value ==False and kwargs.get(key):
                del instructions[key]
            else:
                instructions[key] = self.default_instructions.get(key) or value
        self.update_instruction(instructions)
        return instructions
    def update_instructions_text(self,**kwargs)->dict:
        instructions = get_instructions_dict()
        for key,value in instructions.items():
            if key not in instruction:
               instruction[key]=self.default_instructions.get(key,{"instruction":"","example":"","default":False,"bool":False})
            instructions[key].update(value)
        self.update_instruction(instructions)
        return instructions
    def make_instructions_reference(self):
        instructions = self.default_instructions
        instructions.update(self.get_instructions_dict())
        instructions_bools = {key:False if not self.get_instructions_dict().get(key) else True for key in instructions.keys()}
        instructions_text = self.get_instructions_text()
        return {
            "instructions_bools":instructions_bools,
            "instructions_text_values":instructions_text,
            "instructions_text":get_instructions_text(),
            "text":instructions_text}
        
    def find_instruction_from_list(self,search_key):
        for key,inst in self.instructions_bool_list.items():
            if key==search_key:
                return inst
    def update_instructions_text_values(self,**kwargs)->dict:
        for key,value in kwargs.items():
            parameter_text_values[parameter] = self.default_instructions[parameter]['instruction'] if is_bool(parameters[parameter]) else  parameters[parameter]
        return parameter_text_values
    def get_instructions_text(self)->dict:
        """
        Retrieves instructions for the conversation.

        Returns:
            None
        """
        instructions = ""
        example_format={}
        instruction_dict=self.get_instructions_dict()

        instructions = "your response is expected to be in JSON format with the keys as follows:\n\n"
        i=0
        for key,value in instruction_dict.items():
            instructions+=f"{i}) {key} - {value.get('instruction')}\n"
            example_format[key]=value.get('example')
            i+=1
        instructions += '\nbelow is an example of the expected json dictionary response format, with the default inputs:\n' + str(example_format).replace("'",'"')
        return instructions

    def update_instructions(self,instruction_number:int,**kwargs)->None:
        instruction_display,instruction_bools,instructions_text_values,instructions_text=self.get_instructions_values(**kwargs)
        self.instructions[instruction_number]={'instruction_display':instruction_display,'instruction_bools':instruction_bools,"instructions_text":instructions_text_values,"text":instructions_text}




from ..dependencies import *
def get_output_response(self,model:str="None",title:str="None",request:str="None",response:str="None",output_files:str="None")->str:
    """
    A helper method that formats the output string of a response in a clear and readable manner.
    It includes information about the model, title, user request, and AI response.
    """
    self.logger.info(f"get_output_response..")
    model = f"#MODEL#{self.get_new_line(1)}{self.last_model}{self.get_new_line(2)}"
    title = f"#TITLE#{self.get_new_line(1)}{str(title)}{self.get_new_line(2)}"
    request = f"#USER REQUEST#{self.get_new_line(1)}{str(request)}{self.get_new_line(2)}"
    response = f"#RESPONSE#{self.get_new_line(2)}{str(response)}{self.get_new_line(2)}"
    output_files = f"#OUTPUT FILES#{self.get_new_line(1)}{str(output_files)}{self.get_new_line(2)}"
    return f"{model}{title}{request}{response}{output_files}"

def update_text_with_responses(self)->None:
    """
    gets the latest response from the AI, formats it using the get_output_response helper method,
    and updates the GUI's '-RESPONSE-' key with the formatted string.
    """
    self.logger.info(f"update_text_with_responses..")
    output_keys = []
    self.last_prompt_content,response_content = get_data_from_response(self.current_output)
    self.last_response_content = response_content
    self.last_request= self.last_prompt_content.get('prompt')
    self.last_file_path= get_file_path(self.last_response_content)
    self.last_title= get_title(self.last_response_content)
    self.last_model = get_model(self.last_response_content)
    self.last_api_response = self.last_response_content.get('api_response')
    self.window_mgr.update_value(key=text_to_key('title input'),value=self.last_title)
    #self.database_output_file = generate_query_from_recent_response(file_path=self.last_file_path)
    if self.last_api_response:
        response_content=self.last_api_response
        self.delegate_instruction_text()
    self.window_mgr.update_value('-RESPONSE-',self.get_output_response(model=self.last_model,
                                                                       title=self.last_title,
                                                                       request=self.last_request,
                                                                       response=response_content,
                                                                       output_files=self.last_file_path))
